%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            One dimensional fluid flow in a straight channel             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear 
close all
tic

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Solver specifications
maxits = 20000;
N = 100;                                    % Number of scalar nodal points 
runiterationwise = 0;             % Plots the profiles after each iteration
plotinitialprofiles = 0;                         % Plot the initial guesses
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initial guesses

p_out = 1e5;        % Pa
u_in = 1e-3;        % m/s
p_guess = 1.5e5;    % Pa
u_guess = 1.5e-3;   % m/s

% Creating a linear profile for the initial guess of the pressure
pprofile = [linspace(p_guess,p_out,N+1), p_out];
p_circ = pprofile(2:end-1);

% Creating a linear profile for the initial guess of the velocity
uprofile = linspace(u_in,u_guess,N+1);
% Placing the velocities in the staggered grid
u_circ = 0.5*(uprofile(2:end)+uprofile(1:end-1));

alphau = 1;        % Under-relaxation of the velocity
alpha_p = 0.05;     % Under-relaxation of the pressure     
% 1 corresponds to no under-relaxation 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parameters and system specifications
l = 3;              % Channel length [m]

x_0 = 0;            % Channel start [m]
x_N = l;            % Channel end [m]
mu = 8.90 * 10^-4;  % Viscosity [Pa s]  

sys_width = 1;      % Height of the channel is set to unity 
                    % for the one dimensional model

del_x = x_N/N;      % Width of the control volume [m]
A = del_x*sys_width;% Cross-sectional area [m^2]


rho = 1e3;          % Density [kg/m^3]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation

p_corr = zeros(1, N);
p_new = zeros(1, N);

u_star = zeros(1, N);
u_corr = zeros(1, N);
u_new = zeros(1, N);

F_e = zeros(1, N);
F_w = zeros(1, N);
D = mu/del_x;

U = zeros(N, N);
bu = zeros(1, N);

T = zeros(N, N);
beta = zeros(1, N);
au = zeros(1, N);

xu_plot = linspace(x_0, x_N, N+1); % staggered grid
xp_plot = linspace(x_0+del_x/2, x_N+del_x/2, N+1); 

if plotinitialprofiles == 1
    figure
    plot(xu_plot, [u_in, u_circ])
    hold on
    plot(xu_plot(1:2), [u_in, u_circ(1)],'r')
    title('Initial guess $u$', 'interpreter', 'latex')
    figure
    plot(xp_plot, [p_circ,p_out])
    hold on
    plot(xp_plot(end-1:end), [p_circ(end),p_out],'r')
    title('Initial guess $p$', 'interpreter', 'latex')
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% While loop
conv = 0; 
it = 1;
while conv == 0
    
    %% Solve momentum equation
    % Calculation of coefficients F_e:
    for i = 1:length(u_circ)-1
        F_e(i) = rho*1/2*(u_circ(i+1)+u_circ(i));
    end %for
    F_e(end) = rho*1/2*(u_circ(end)+u_circ(end-1)); % F_e = F_w
   
    % Calculation of coefficients F_w:
    F_w(1) = rho*1/2*(u_circ(1)+u_in);
    for i = 2:length(u_circ)
        F_w(i) = rho*1/2*(u_circ(i)+u_circ(i-1));
    end %for
  
    % u_2 (u(1))
    U(1,1) = 4*D*A +  max(0, -F_e(1)*A) + max(F_w(1)*A,0)...
        + F_e(1)*A - F_w(1)*A; 
    U(1,2) = -2*D*A - max(0, -F_e(1)*A);
    bu(1) = ( 2*D*A +  max(F_w(1)*A, 0))*u_in ...
        - A*(p_circ(2) - p_circ(1));

        
    % u_centers
    for j = 2:length(u_circ)-1
        U(j,j) =  4*D*A +  max(0, -F_e(j)*A) + max(F_w(j)*A, 0) ...
            + F_e(j)*A - F_w(j)*A ; 
        U(j,j+1) = -2*D*A- max(0, -F_e(j)*A);  
        U(j,j-1) =  -2*D*A- max(F_w(j)*A, 0);
        bu(j) =  - (p_circ(j+1) - p_circ(j))*A;

        
    end %for

    % u_n+1 (u(end))
    U(end,end) =  4*D*A + max(F_w(end)*A, 0) ...
        + F_e(end)*A - F_w(end)*A ;
    U(end,end-1) = -2*D*A -max(F_w(end)*A, 0);
    bu(end) = - (p_out - p_circ(end))*A;
        
    % Matrix inversion
    u_star = U\bu';
        
    %% Solve pressure correction equation
    % a^center-coefficients in the momentum equation
	au = diag(U);

       
    % p_1
    T(1,1) = rho*A/au(1);
    T(1,2) = - rho*A/au(1); 
    beta(1) =  rho*(-u_star(1) + u_in);
    
    % p_centers
    for j = 2:length(p_corr)-1 
        T(j,j) = rho*A*(1/au(j) + 1/au(j-1));
        T(j,j+1) = - rho*A/au(j); 
        T(j,j-1) = - rho*A/au(j-1); 
        beta(j) =  rho*(-u_star(j) + u_star(j-1));
    end %for

    % p_N
    T(end,end) = rho*A*(1/au(end)+1/au(end-1));
    T(end,end-1) =  -  rho*A/au(end); 
    beta(end) = rho*(-u_star(end) + u_star(end-1));
    
    % Matrix inversion
    p_corr = T\beta';
   
    %% Velocity correction
    for j = 1:length(p_corr)-1
        u_corr(j) = - A/au(j)*(p_corr(j+1)-p_corr(j)); 
    end %for
    u_corr(end) = - A/au(end)*(-p_corr(end));  
    % pressure correction is zero for the known outlet pressure 

    %% Under-relaxation
    % Pressure 
    p_new = p_circ + alpha_p* p_corr';
    
    % Under-relaxation of u
    u_new = alphau*(u_star' + u_corr) + (1-alphau)*u_circ;

    %% Check convergence 
    if isnan(rcond(U)) || isnan(rcond(T))
        fprintf('Stopped due to singularity in matrix\n')
        fprintf('RCOND velocity: %e \nRCOND pressure: %e\n',...
            rcond(U), rcond(T))
        fprintf('Problem occured after %d iterations\n', it-1)
        return
    end %if

    c1 = 1/u_in*sqrt((U*u_star-bu')'*(U*u_star-bu')); % coefficient summed
    c3 = abs(sum(beta)); % continuity 
    c4 =  1/u_in*max(abs(u_circ - u_star')); % change from last iteration
    
    c1_lim = 10^-6;
    c3_lim = 10^-6;
    c4_lim = 10^-6;
    
    c1_diff = c1-c1_lim;
    c3_diff = c3-c3_lim;
    c4_diff = c4-c4_lim;

    if  (c1 < c1_lim) && (c3 < c3_lim) && (c4 < c4_lim) || (it == maxits)
        conv = 1; % While loop is stopped
        if (it == maxits)
            fprintf('Stopped at max iterations (%d)\n',it);
        else
            fprintf('Solution converged after %d iterations\n',it);
        end %if

        fprintf('c1\tMomentum residual\t\t%.2e\tLimit: %.2e\n',c1,c1_lim);
        fprintf('c3\tPressure correction\t\t%.2e\tLimit: %.2e\n',c3,c3_lim);
        fprintf('c4\tDiff. last iteration\t%.2e\tLimit: %.2e\n',c4,c4_lim);
        
        if max([c1_diff c3_diff c4_diff])== c1_diff
            fprintf('Limiting criteria is c1\tMomentum residual\n')
        elseif max([c1_diff c3_diff c4_diff])== c3_diff
            fprintf('Limiting criteria is c3\tPressure correction\n')
        elseif max([c1_diff c3_diff c4_diff])== c4_diff
            fprintf('Limiting criteria is c4\tDiff. last iteration\n')
        end %if
        
        
    else
        u_circ = u_new; % Not converged, updated variables.
        p_circ = p_new; 
        it = it + 1;

        
    end %if
    if runiterationwise == 1 || conv == 1 
        % For iterationwise plotting and for when the model is stopped
        % Plot after each iteration and close before proceding to the next
        
        u_new_plot = [u_in u_new];
                  
        % Discretied x-node points
        p_plot = [p_new p_out];
        p_corr_plot = [p_corr' 0];

        fu = figure;
        plot(xu_plot, u_new_plot)
        s = sprintf('Plot of $u^{new}$ after %d iterations', it-1 );
%         f = title(s);
%         set(f, 'interpreter', 'latex', 'fontsize', 16)
        set(gca,'TickLabelInterpreter','latex')
        xlabel('$x$-direction [m]', 'interpreter', 'latex')
        xlim([0,3])
        ylabel('Velocity $u$, [m/s]', 'interpreter', 'latex')
%         set(fu, 'Position', [5,217,414.6667,420]); 
        %[left bottom width height]
        saveas(gcf,'unew1D.png')


        fp = figure;
        plot(xp_plot, p_plot)
        s = sprintf('Plot of $p^{new}$ after %d iterations', it-1 );
%         f = title(s);
%         set(f, 'interpreter', 'latex', 'fontsize', 16)
        set(gca,'TickLabelInterpreter','latex')
        xlabel('$x$-direction [m]', 'interpreter', 'latex')
        xlim([0,3])
        ylabel('Pressure $p$, [Pa]', 'interpreter', 'latex')
%         set(fp, 'Position', [419.6667,217,434.6667,420]); 
        % [left bottom width height]
        saveas(gcf,'pnew1D.png')
        
        fpcorr = figure;
        plot(xp_plot, p_corr_plot)
        s = sprintf('Plot of $p^{corr}$ after %d iterations', it-1 );
%         f = title(s);
%         set(f, 'interpreter', 'latex', 'fontsize', 16)
        set(gca,'TickLabelInterpreter','latex')
        xlabel('$x$-direction [m]', 'interpreter', 'latex')
        xlim([0,3])
        ylabel('Pressure correction $p$, [Pa]', 'interpreter', 'latex')
%         set(fpcorr, 'Position', [855,217.6667,424,422.6667]); 
        % [left bottom width height]
        saveas(gcf,'pcorr1D.png')

        if conv ~= 1 % if not converged
           pause
           close all
        end %if
    end % if
  
end %while

toc

