%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Two dimensional fluid flow in a straight channel, dimensionless      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
clc
close all
tic

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Solver specifications
maxits = 100000;  % Maximum number of iterations, stop if iterations exceed
N = 88;                      % Number of scalar nodal points in x-direction
M = 18;                      % Number of scalar nodal points in y-direction 
runiterationwise = 0;             % Plots the profiles after each iteration
plotinitialprofiles = 0;                         % Plot the initial guesses
solvvel = true;                                      % Solve for v-velocity
contplots = false;           % Show plots of continuity + cont_x and cont_y
v_out_zero = false;                % Use v_out = zero as boundary condition

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% System specifications
m = M - 1;                      % Number of y-velocity nodes in y-direction
L = 22;                                                    % Channel length
h = 1;                                                     % Channel height
D_hyd = 4*h*1/(1+1+h+h);           % Hydraulic diameter for Reynolds number

x_0 = 0;                                % Defining the domain using x and y
x_N = L;
y_0 = 0;
y_M = h;

mu_true = 8.90 * 10^-4;                                % Viscosity of water

del_z_true = 1;                                              % System depth
del_x_true = x_N/N;                                  % Control volume width
del_y_true = y_M/M;                                 % Control volume height
A_x_true = del_y_true*del_z_true;     % Cross-sectional area in x-direction
A_y_true = del_x_true*del_z_true;     % Cross-sectional area in y-direction

rho_true = 997;                                          % Density of water
u_in_true = 0.0005;                                      % Inlet u-velocity 
g_x = 0;                                                   % No gravitation
g_y = 0;                                                   % No gravitation


Re = rho_true*D_hyd*u_in_true/mu_true;                    % Reynolds number

p_atm = 101325;                           % Atmospheric presssure at outlet
p_out_tilde = 0;                                        % Adjusted pressure
p_out = ones(1,M)*p_out_tilde;                    % Outlet pressure profile

alpha_u = 0.01;                             % Under-relaxation factor for u 
alpha_v = 0.01;                             % Under-relaxation factor for v
alpha_p = 0.02;                             % Under-relaxation factor for p

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dimensionless parameters
mu = 1;                                           % Dimensionless viscosity
rho = 1;                                            % Dimensionless density
del_x = del_x_true/D_hyd;              % Dimensionless control volume width
del_y = del_y_true/D_hyd;             % Dimensionless control volume height
A_x = A_x_true/D_hyd^2; % Dimensionless cross-sectional area in x-direction
A_y = A_y_true/D_hyd^2; % Dimensionless cross-sectional area in y-direction
D_x = 1/Re*mu/del_x;   % Dimensionless diffusion conductance in x-direction
D_y = 1/Re*mu/del_y;   % Dimensionless diffusion conductance in y-direction
u_in = 1;                                                % Inlet u-velocity
v_in = 0;                                                % Inlet u-velocity
u_guess = 1.0; %                             % Initial guess for u-velocity
v_guess = 0.0; %                             % Initial guess for v-velocity
u_circ = ones(1,M*N)*u_guess;         % Initial guess vector for u-velocity
v_circ = ones(1,m*N)*v_guess;         % Initial guess vector for v-velocity
p_guess = 0/(rho_true*u_in_true^2);            % Initial guess for pressure    
p_circ_vector = linspace(p_guess,p_out_tilde,N)';     % Linear profile from 
                                           % guess to known outlet pressure
p_circ = zeros(M*N,1); 
for j = 1:M    % Filling in initial pressure vector with the linear profile 
    p_circ((j-1)*N+1:j*N) = p_circ_vector; 
end %for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialization of solution vectors
p_corr = zeros(1, M*N);                               % Pressure correction
p_new = zeros(1, M*N);                                       % New pressure

u_star = zeros(1, M*N);                 % u-velocity after matrix inversion
u_corr = zeros(1, M*N);                             % u-velocity correction
u_new = zeros(1, M*N);                                     % New u-velocity
U = zeros(M*N, M*N);                        % u-velocity coefficient matrix
bu = zeros(1, M*N);                         % u-velocity source term vector

v_star = zeros(1, m*N);                 % v-velocity after matrix inversion
v_corr = zeros(1, m*N);                             % v-velocity correction
v_new = zeros(1, m*N);                                     % New v-velocity
V = zeros(m*N, m*N);                        % v-velocity coefficient matrix
bv = zeros(1, m*N);                         % v-velocity source term vector

F_xe = zeros(1, M*N);                  % Convective mass flux per unit area
F_xw = zeros(1, M*N);
F_xn = zeros(1, M*N);
F_xs = zeros(1, M*N);
F_ye = zeros(1, m*N);
F_yw = zeros(1, m*N);
F_yn = zeros(1, m*N);
F_ys = zeros(1, m*N);
                        
T = zeros(M*N, M*N);               % Pressure correction coefficient matrix                                   % for pressure
beta = zeros(1, M*N);              % Pressure correction source term vector

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plots of initial guesses
if plotinitialprofiles == true
    f111 = figure;
    surf(linspace(x_0+del_x/2, x_N+del_x/2, N+1),...
             linspace(y_0+del_y/2, y_M-del_y/2, M),...
             [global2matrix(p_circ,N,M) p_out']);             % surf(x,y,z)
    s = sprintf('Initial guess $p_{circ}$');
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Pressure $p$, [Pa]', 'interpreter', 'latex')

    u_circ_carthesian = ones(M,N+1)*u_guess;
    u_circ_carthesian(:,1) = u_in;

    f122 = figure;
    surf(linspace(x_0, x_N, N+1),...                          % surf(x,y,z)
        linspace(y_0+del_y/2, y_M-del_y/2, M),u_circ_carthesian);         
    s = sprintf('Initial guess $u_{circ}$');
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Velocity $u$, [m/s]', 'interpreter', 'latex')

    v_circ_carthesian = ones(M+1,N+1)*v_guess;
    v_circ_carthesian(1,:) = 0;
    v_circ_carthesian(M+1,:) = 0;
    v_circ_carthesian(:,1) = 0;   
    
    f133 = figure;
    surf(linspace(-x_0-del_x/2, x_N+del_x/2, N+1),...
        linspace(y_0, y_M, M+1),v_circ_carthesian);           % surf(x,y,z)
    % set(f,'edgecolor','none')
    s = sprintf('Initial guess $v_{circ}$');
    f = title(s);
    set(f, 'interpreter', 'latex', 'fontsize', 16)
    set(gca,'TickLabelInterpreter','latex')
    xlabel('$x$-direction [m]', 'interpreter', 'latex')
    ylabel('$y$-direction [m]', 'interpreter', 'latex')
    zlabel('Velocity $v$, [m/s]', 'interpreter', 'latex')
pause 
close all
end % if

% Defining x and y points for the staggered grid for plotting
xu_plot = linspace(x_0, x_N, N+1); 
yu_plot = [0,linspace(y_0+del_y/2, y_M-del_y/2, M),h];
xv_plot = [0,linspace(x_0+del_x/2, x_N-del_x/2, N)]; 
yv_plot = linspace(y_0, y_M, M+1);
xp_plot = linspace(x_0+del_x/2, x_N+del_x/2, N+1) + del_x_true/2;   
yp_plot = linspace(y_0+del_y/2, y_M-del_y/2, M) + del_y_true/2;

%% Specifications before iteration
if solvvel == false  % Not solve for v-velocity
    v_out_zero = false; % Turn off outlet boundary condition for v
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% While loop
conv = 0;                            % 0 is not converged, 1 when converged
it = 1;                                             % The current iteration

% Coefficients in matrix, example:
% sP_coeff is part of the a_P-coefficient at the diagonal position in the 
% matrix, while S_coeff is the coefficient in the matrix for the south node 
while  conv == 0 % it<=maxits %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Generation of F
    %Generation of F_x:
    for i = 1:M*N 
        
        etest = mod(i, N) == 0;
        wtest = mod(i-1, N) == 0;
        ntest = M*N - (N - 1) <= i && i <= N*M  ;
        stest = 1 <= i && i <= N  ;
        
        % Northeastern corner
        if etest == true && ntest == true
            F_xe(i) = rho/2*(u_circ(i-1)+u_circ(i));   % F_xe = F_xw
            F_xn(i) = 0;                               % v_NorthWall = 0;
            
            F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
            F_xs(i) = rho/2*v_circ(i-N);
        
        % Southeastern corner
        elseif etest == true && stest == true
            F_xe(i) = rho/2*(u_circ(i-1)+u_circ(i));   % F_xe = F_xw
            F_xs(i) = 0;                               % v_SouthWall = 0;  
            
            F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
            F_xn(i) = rho/2*v_circ(i);
        
        % Northwestern corner
        elseif wtest == true && ntest == true
            F_xw(i) = rho/2*(u_in+u_circ(i));          % Inlet
            F_xn(i) = 0;                               % v_NorthWall = 0;

            F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
            F_xs(i) = rho/2*(v_circ(i-N) + v_circ(i-N+1));     
        
        % Southwestern corner
        elseif wtest == true && stest == true
            F_xw(i) = rho/2*(u_in+u_circ(i));          % Inlet
            F_xs(i) = 0;                               % v_SouthWall = 0;
                  
            F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
            F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
        
        % At eastern boundary (x = L)
        elseif etest == true && ntest == false && stest == false 
            F_xe(i) = rho/2*(u_circ(i-1)+u_circ(i));   % F_xe = F_xw
            
            F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
            F_xn(i) = rho/2*v_circ(i); 
            F_xs(i) = rho/2*v_circ(i-N); 
        
        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false 
            F_xw(i) = rho/2*(u_in+u_circ(i));          % Inlet
            
            F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
            F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
            F_xs(i) = rho/2*(v_circ(i-N) + v_circ(i-N+1));
        
        % At northern boundary (y = h)
        elseif ntest == true && etest == false && wtest == false 
           F_xn(i) = 0;                                % v_NorthWall = 0;
           
           F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
           F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
           F_xs(i) = rho/2*(v_circ(i-N) + v_circ(i-N+1));
      
        % At southern boundary (y = 0) 
        elseif stest == true && etest == false && wtest == false  
            F_xs(i) = 0;                               % v_SouthWall = 0;
            
            F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
            F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
            F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));    
        
        %Not at any boundary
        else
            F_xe(i) = rho/2*(u_circ(i+1)+u_circ(i));
            F_xw(i) = rho/2*(u_circ(i-1)+u_circ(i)); 
            F_xn(i) = rho/2*(v_circ(i) + v_circ(i+1));
            F_xs(i) = rho/2*(v_circ(i-N) + v_circ(i-N+1));
        
        end % if
        
        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
        
    end %for
    
    %Generation of F_y:
    for i = 1:m*N % Global indexing system 
    
        % Eastern boundary requires no special treatment (x = L)
        wtest = mod(i-1, N) == 0;
        ntest = m*N - (N - 1) <= i && i <= m*N  ;
        stest = 1 <= i && i <= N  ;
                 
        % Northwestern corner
        if wtest == true && ntest == true
            F_yw(i) = rho*u_in;                        % inlet
            F_yn(i) = rho/2*v_circ(i);                 % v_NorthWall = 0;
        
            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_ys(i) = rho/2*(v_circ(i) + v_circ(i-N));
           
        % Southwestern corner
        elseif wtest == true && stest == true
            F_yw(i) = rho*u_in;                        % inlet
            F_ys(i) = rho/2*v_circ(i);                 % v_SouthWall = 0;
        
            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_yn(i) = rho/2*(v_circ(i) + v_circ(i+N));
              
        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false 
            F_yw(i) = rho*u_in;                        % inlet

            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_yn(i) = rho/2*(v_circ(i) + v_circ(i+N));
            F_ys(i) = rho/2*(v_circ(i) + v_circ(i-N));
          
        % At northern boundary (y = h)
        elseif ntest == true && wtest == false 
            F_yn(i) = rho/2*v_circ(i);                 % v_NorthWall = 0;
      
            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_yw(i) = rho/2*(u_circ(i-1) + u_circ(i-1+N));
            F_ys(i) = rho/2*(v_circ(i) + v_circ(i-N));
                           
        % At southern boundary (y = 0) 
        elseif stest == true && wtest == false  
            F_ys(i) = rho/2*v_circ(i);                 % v_SouthWall = 0;
            
            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_yw(i) = rho/2*(u_circ(i-1) + u_circ(i-1+N));
            F_yn(i) = rho/2*(v_circ(i) + v_circ(i+N));
             
        %Not at any boundary
        else
            F_ye(i) = rho/2*(u_circ(i) + u_circ(i+N));
            F_yw(i) = rho/2*(u_circ(i-1) + u_circ(i-1+N));
            F_yn(i) = rho/2*(v_circ(i) + v_circ(i+N));
            F_ys(i) = rho/2*(v_circ(i) + v_circ(i-N));
        
        end % if
        
        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
        
    end % for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% u-velocity 
    for i = 1:M*N % Global indexing system 
        
        etest = mod(i, N) == 0;
        wtest = mod(i-1, N) == 0;
        ntest = M*N - (N - 1) <= i && i <= N*M  ;
        stest = 1 <= i && i <= N  ;
        
        % Northeastern corner
        if etest == true && ntest == true
            % At eastern boundary (x = L) 
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff =  F_xe(i)*A_x;   
            
            bu(i) =  -(p_out(end)-p_circ(i))*A_x; 

            % At northern boundary 
            nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y) + 2*D_y*A_y;          
            %wall shear stress 
            
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
  
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;
        
        % Southeastern corner
        elseif etest == true && stest == true
            % At eastern boundary (x = L) 
            E_coeff = -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff =  F_xe(i)*A_x;   
            
            bu(i) =  -(p_out(1)-p_circ(i))*A_x; 

            % At southern boundary (y = 0)
            sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;         
            %wall shear stress
            
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;
        
        % Northwestern corner
        elseif wtest == true && ntest == true
            % At western boundary (x = 0) 
            wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

            % At northern boundary 
            nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y)+ 2*D_y*A_y;          
            %wall shear stress
            
            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x ...
                +(max(F_xw(i)*A_x,0) + D_x*A_x)*u_in; 
            
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;
            
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;          
        
        % Southwestern corner
        elseif wtest == true && stest == true
            % At western boundary (x = 0) 
            wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

            % At southern boundary (y = 0)
            sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;         
            %wall shear stress
            
            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x...
                +(max(F_xw(i)*A_x,0) + D_x*A_x)*u_in; 
            
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;

            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;
  
        % At eastern boundary (x = L)
        elseif etest == true && ntest == false && stest == false 
            % At eastern boundary (x = L) 
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff =  F_xe(i)*A_x;   
            
            bu(i) =  -(p_out(floor((i-1)/N)+1)-p_circ(i))*A_x;
   
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;
  
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;          
        
        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false 
            % At western boundary (x = 0) 
            wP_coeff = max(F_xw(i)*A_x,0) + D_x*A_x - F_xw(i)*A_x;

            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x ...
                +(max(F_xw(i)*A_x,0) + D_x*A_x)*u_in; 
            
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;
            
            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;
  
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;
        
        % At northern boundary (y = h)
        elseif ntest == true && etest == false && wtest == false 
            % At northern boundary 
            nP_coeff = F_xn(i)*A_y + max(0,-F_xn(i)*A_y)+ 2*D_y*A_y; 
            %wall shear stress
           
            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
              
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;           
      
        % At southern boundary (y = 0) 
        elseif stest == true && etest == false && wtest == false  
            % At southern boundary (y = 0)
            sP_coeff = -F_xs(i)*A_y +max(F_xs(i)*A_y,0)+ 2*D_y*A_y;        
            %wall shear stress
            
            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
            
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;          
        
        %Not at any boundary
        else 
            bu(i) = -(p_circ(i+1)-p_circ(i))*A_x;
            E_coeff =  -max(0,-F_xe(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_xe(i)*A_x;
            U(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_xw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_xw(i)*A_x;          
            U(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_xn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_xn(i)*A_y;
            U(i, i+N) = N_coeff;
  
            S_coeff = -max(F_xs(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_xs(i)*A_y;
            U(i, i-N) = S_coeff;
            
        end % if
        
        % Filling in the rest of the matrix, adding all point coefficients
        U(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;
        
        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
       
    end %for
 
    u_star = U\bu';      
     
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% v-velocity
    for i = 1:m*N % Global indexing system 
        bv(i) = -(p_circ(i+N)-p_circ(i))*A_y + rho*g_y*del_y*A_y;
        
        etest = mod(i, N) == 0;
        wtest = mod(i-1, N) == 0;
        ntest = m*N - (N - 1) <= i && i <= m*N  ;
        stest = 1 <= i && i <= N  ;
        
        % Northeastern corner
        if etest == true && ntest == true
            
            % At eastern boundary (x = L)
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff =  F_ye(i)*A_x;     
            
            if v_out_zero == true
                eP_coeff = eP_coeff + 1e+30; 
            end %if  
            
            % At northern boundary 
            nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y;  
  
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff;         
   
        % Southeastern corner
        elseif etest == true && stest == true
            
            % At eastern boundary (x = L)
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff =  F_ye(i)*A_x;  
            if v_out_zero == true
                eP_coeff = eP_coeff + 1e+30; 
            end %if  
            % At southern boundary (y = 0), 
            sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y; 
              
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;           
            
        % Northwestern corner
        elseif wtest == true && ntest == true

            % At western boundary (x = 0)
            wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;
    
            % At northern boundary 
            nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y ;  
         
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff;     
        
        % Southwestern corner
        elseif wtest == true && stest == true
            
            % At western boundary (x = 0)
            wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;
    
            % At southern boundary (y = 0), 
            sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y;  
                      
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;
            
        % At eastern boundary (x = L)
        elseif etest == true && ntest == false && stest == false 

            % At eastern boundary (x = L)
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff =  F_ye(i)*A_x;  
            if v_out_zero == true
                eP_coeff = eP_coeff + 1e+30;
            end %if  
            
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;  
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff;   

        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false 

            % At western boundary (x = 0)
            wP_coeff = - F_yw(i)*A_x + max(F_yw(i)*A_x,0) + 2*D_x*A_x;
            
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff; 
        
        % At northern boundary (y = h)
        elseif ntest == true && etest == false && wtest == false 
    
            % At northern boundary 
            nP_coeff = F_yn(i)*A_y + max(0, -F_yn(i)*A_y) + D_y*A_y ;   
         
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff;      
      
        % At southern boundary (y = 0) 
        elseif stest == true && etest == false && wtest == false  
 
            % At southern boundary (y = 0), 
            sP_coeff = -F_ys(i)*A_y + max(F_ys(i)*A_y,0) + D_y*A_y;
                      
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;
            
        %Not at any boundary
        else 
            
            E_coeff =  -max(0,-F_ye(i)*A_x) - D_x*A_x;
            eP_coeff = -E_coeff + F_ye(i)*A_x;
            V(i, i+1) = E_coeff;
            
            W_coeff =  -max(F_yw(i)*A_x,0) - D_x*A_x;
            wP_coeff = -W_coeff - F_yw(i)*A_x;     
            V(i, i-1) = W_coeff;
            
            N_coeff = -max(0,-F_yn(i)*A_y) - D_y*A_y;    
            nP_coeff = -N_coeff + F_yn(i)*A_y;
            V(i, i+N) = N_coeff;
            
            S_coeff = -max(F_ys(i)*A_y,0) - D_y*A_y;
            sP_coeff =  -S_coeff - F_ys(i)*A_y;
            V(i, i-N) = S_coeff;
            
        end % if
        
        % Filling in the rest of the matrix, adding all point coefficients
        V(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;
        
        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
           
    end % for
    v_star = V\bv';
    if solvvel == false
        v_star = zeros(length(v_star),1);
    end %if
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Pressure correction 
    au = diag(U);                    % a^center-coefficients for u-velocity
    av = diag(V);                    % a^center-coefficients for v-velocity

    for i = 1:M*N % Global indexing system 
        
        etest = mod(i, N) == 0;
        wtest = mod(i-1, N) == 0;
        ntest = M*N - (N - 1) <= i && i <= N*M  ;
        stest = 1 <= i && i <= N  ;
        
        % Northeastern corner
        if etest == true && ntest == true
            
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                + A_y*v_star(i-N));
            
            % At eastern boundary (x = L)
            eP_coeff = rho*A_x^2/au(i);  
                                         
            % At northern boundary 
            nP_coeff = 0 ;  

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            S_coeff = -rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;            
        
        % Southeastern corner
        elseif etest == true && stest == true
            
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                -A_y*v_star(i));
            
            % At eastern boundary (x = L)
            eP_coeff = rho*A_x^2/au(i);  
   
            % At southern boundary (y = 0)
            sP_coeff = 0; 

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

        % Northwestern corner
        elseif wtest == true && ntest == true
            
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in  ...
                + A_y*v_star(i-N));
            
            % At western boundary (x = 0)
            wP_coeff = 0;

            % At northern boundary 
            nP_coeff = 0 ;  
           
            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            S_coeff = -rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;        
        
        % Southwestern corner
        elseif wtest == true && stest == true
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in ...
                -A_y*v_star(i));
            
            % At western boundary (x = 0)
            wP_coeff = 0;

            % At southern boundary (y = 0)
            sP_coeff = 0; 

            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

        % At eastern boundary (x = L)
        elseif etest == true && ntest == false && stest == false 
            
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                -A_y*v_star(i) + A_y*v_star(i-N)); 
            
            % At eastern boundary (x = L)
            eP_coeff = rho*A_x^2/au(i);  

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

            S_coeff = -rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;     
            
        % At western boundary (x = 0)
        elseif wtest == true && ntest == false && stest == false 
            
            beta(i) =  rho*(-A_x*u_star(i) +A_x*u_in ...
                -A_y*v_star(i) + A_y*v_star(i-N));
            
            % At western boundary (x = 0)
            wP_coeff = 0;

            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

            S_coeff =- rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;   
            
        % At northern boundary (y = h)
        elseif ntest == true && etest == false && wtest == false 
            
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                + A_y*v_star(i-N));
            
            % At northern boundary 
            nP_coeff = 0 ;  

            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            S_coeff = -rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;       
            
        % At southern boundary (y = 0) 
        elseif stest == true && etest == false && wtest == false  
            
            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1)...
                -A_y*v_star(i));
            
            % At southern boundary (y = 0)
            sP_coeff = 0; 

            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

        %Not at any boundary
        else 

            beta(i) = rho*(-A_x*u_star(i) +A_x*u_star(i-1) ...
                -A_y*v_star(i) + A_y*v_star(i-N)); 
            
            E_coeff = -rho*A_x^2/au(i);
            eP_coeff = -E_coeff ;
            T(i, i+1) = E_coeff;

            W_coeff =  -rho*A_x^2/au(i-1);
            wP_coeff = -W_coeff;
            T(i, i-1) = W_coeff;

            N_coeff = -rho*A_y^2/av(i);
            nP_coeff = -N_coeff;
            T(i, i+N) = N_coeff;

            S_coeff = -rho*A_y^2/av(i-N);
            sP_coeff =  -S_coeff;
            T(i, i-N) = S_coeff;                
            
        end % if
        
        % Filling in the rest of the matrix, adding all point coefficients
        T(i,i) = wP_coeff + eP_coeff + nP_coeff + sP_coeff;
                    
        etest = false;
        wtest = false;
        ntest = false;
        stest = false;
    end % for
    p_corr = T\beta';

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Velocity correction

    for j = 1:length(p_corr)
        if mod(j, N) == 0                                % eastern boundary
            u_corr(j) =  - A_x/au(j)*(-p_corr(j)); 
                    % pressure correction is zero for known outlet pressure 
        else
            u_corr(j) =  - A_x/au(j)*(p_corr(j+1)-p_corr(j));
        end % if
    end %for
    
    for k = 1:length(p_corr)-N
            v_corr(k) =  - A_y/av(k)*(p_corr(k+N)-p_corr(k));
    end %for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Under-relaxation 
    
    p_new = p_circ + alpha_p* p_corr;
    u_new = alpha_u*(u_star' + u_corr) + (1-alpha_u)*u_circ;
    v_new = alpha_v*(v_star' + v_corr) + (1-alpha_v)*v_circ;
    
    if solvvel == false
        v_new = zeros(1,length(v_new));
        annoSolvvel = '$v$-velocity: Not solved';
    else 
        annoSolvvel = '$v$-velocity: Solved';
    end %if
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Check convergence 
    % Make sure there are no mistakes in the matrix operations above
    if ~isvector(u_new) || ~isvector(p_new) || ~isvector(p_new) 
        fprintf('u_new - %dx%d\n',size(u_new,1),size(u_new,2))
        fprintf('v_new - %dx%d\n',size(v_new,1),size(v_new,2))
        fprintf('p_new - %dx%d\n',size(p_new,1),size(p_new,2))
        error('Matrix addition gone wrong')        
    end

    if isnan(rcond(U)) || isnan(rcond(V)) || isnan(rcond(T))
%         clc                              % Remove if warnings are desired
        fprintf('Stopped due to singularity in matrix\n')
        fprintf('RCOND u-velocity: %e \nRCOND v-velocity: %e \n',...
            rcond(U), rcond(V))
        fprintf('RCOND pressure: %e\n',rcond(T))
        fprintf('Problem occured after %d iterations\n', it)
        return
    end %if
    
    c1 = 1/u_in*sqrt((U*u_star-bu')'*(U*u_star-bu'));           % residuals 
    c2 = 1/u_in*sqrt((V*v_star-bv')'*(V*v_star-bv'));           % residuals
    c3 = abs(sum(beta));                             % continuity fulfulled 
    c4 = 1/u_in*max(abs(u_circ - u_star')) ;   % change from last iteration
    c5 = 1/u_in*max(abs(v_circ - v_star'))  ;  % change from last iteration


    c1_lim = 10^-8;                                                % Limits
    c2_lim = 10^-8;
    c3_lim = 10^-10;
    c4_lim = 10^-8;
    c5_lim = 10^-8;
    

    if solvvel == false         % Overwrite if v-velocity is not solved for
        c2 = 0;
        c5 = 0;
    end %if
    
    c1_diff = c1-c1_lim;                    % How far away from convergence
    c2_diff = c2-c2_lim;
    c3_diff = c3-c3_lim;
    c4_diff = c4-c4_lim;
    c5_diff = c5-c5_lim;
     
    if  (c1 < c1_lim) && (c2 < c2_lim) && (c3 < c3_lim) && (c4 < c4_lim)...
            && (c5 < c5_lim) || (it == maxits) 
        conv = 1;                                               % Converged
        if (it == maxits)
            fprintf('Stopped at max iterations (%d)\n',it);
        else
            fprintf('Solution converged after %d iterations\n',it);
        end %if
    
        fprintf('c1\tMomentum residual u\t\t%.2e\tLimit: %.2e\n',...
            c1,c1_lim);
        fprintf('c2\tMomentum residual v\t\t%.2e\tLimit: %.2e\n',...
            c2,c2_lim);
        fprintf('c3\tPressure correction\t\t%.2e\tLimit: %.2e\n',...
            c3,c3_lim);
        fprintf('c4\tDiff. last iteration u\t%.2e\tLimit: %.2e\n',...
            c4,c4_lim); 
        fprintf('c5\tDiff. last iteration v\t%.2e\tLimit: %.2e\n',...
            c5,c5_lim);              

        if max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c1_diff
            fprintf('Limiting criteria is c1\tMomentum residual u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c2_diff
            fprintf('Limiting criteria is c2\tMomentum residual v\n')            
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c3_diff
            fprintf('Limiting criteria is c3\tPressure correction\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c4_diff
            fprintf('Limiting criteria is c4\tDiff. last iteration u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c5_diff
            fprintf('Limiting criteria is c5\tDiff. last iteration u\n')            
        end %if
  
    else

        u_circ = u_new  ;                % Not converged, updated variables 
        v_circ = v_new ;                 % Not converged, updated variables 
        p_circ = p_new ;                 % Not converged, updated variables

    end % if
      
    if runiterationwise == 1 || conv == 1
        plot_2D
        if conv == 0 % if not converged
           pause
           close all
        end %if
    end %if
   
    it = it + 1;                              % Update number of iterations
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end %while
toc


