%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Two dimensional fluid flow over a backwards facing step, dimensionless  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear
clc
tic
warning on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Solver specs
maxits = 25000;   % Maximum number of iterations, stop if iterations exceed

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Options
plotiterationwise = false;        % Plots the profiles after each iteration
solvvel = true;                                      % Solve for v-velocity
plotCircVels = false;                              % Plot u_circ and v_circ 
plotCorrVels = false;                              % Plot u_corr and v_corr
showVelociyQuiver = true;                      % Plot velocity quiver plots
plotInitialProfiles = false;                     % Plot the initial guesses
onlyChannel = false;% Turn off the BFS, transform model to straight channel

% Make .gif file of the profiles before convergence is reached:
printSetPlotIt = false;
% Also create a .gif of the u-and v-velocities with their intermediates:
gifIntermediates = false;
% Vector of the iterations for which to save the plots to the .gif files:
itSaves = [1 2 3 4 5 10:10:100 100:100:maxits];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% System specifications
% Specify number of narrow points, leave the rest
N_narrow = 12;  % Number of scalar nodal points in narrow section in x-dir. 
M_narrow = 12;  % Number of scalar nodal points in narrow section in y-dir. 

l = 3;                                              % Narrow channel length 
h = 1;                                              % Narrow channel height
L = 19;                                               % Wide channel length
H = 0.5;                                              % Wide channel height

L_total = l + L;                                     % Total channel length
H_total = h + H;                                     % Total channel height

x_0 = 0;                                % Defining the domain using x and y
x_N = L_total;
y_0 = 0;
y_M = H_total;


if mod(N_narrow,3)~=0 || mod(M_narrow,2)~=0 
    msg = 'Points don''t match dimensions';
    error(msg)
end %if

N_wide = N_narrow*19/3;   % # scalar nodal points in wide section in x-dir. 
M_wide = M_narrow*1/2;    % # scalar nodal points in wide section in y-dir. 

N_total = N_narrow + N_wide;% Total # of scalar nodal points in x-direction 
M_total = M_narrow + M_wide;% Total # of scalar nodal points in y-direction 

m_total = M_total - 1;    % Total number of y-velocity nodes in y-direction
m_wide = M_wide;% Number of y-velocity nodes in y-direction in wide section
m_narrow = M_narrow - 1;% # of y-velocity nodes in y-dir. in narrow section

% Total number of computational points in the domain ...
totalpoints = N_narrow*M_narrow + N_wide*M_total;        %  ... for u and P
totalpoints_v = N_narrow*m_narrow + N_wide*m_total;             % ... for v

D_hyd = 4*h*1/(1+1+h+h);                               % Hydraulic diameter 
mu_true = 8.90 * 10^-4;                                % Viscosity of water

del_z_true = 1;                                              % System depth
del_x_true = L_total/N_total;                        % Control volume width
del_y_true = H_total/M_total;                       % Control volume height
A_x_true = del_y_true*del_z_true;     % Cross-sectional area in x-direction
A_y_true = del_x_true*del_z_true;     % Cross-sectional area in y-direction

rho_true = 997;                                          % Density of water
u_in_true = 0.0005;                                      % Inlet u-velocity 

g_x = 0;                                                   % No gravitation
g_y = 0;                                                   % No gravitation

Re = rho_true*D_hyd*u_in_true/mu_true;                    % Reynolds number

p_atm = 101325;                           % Atmospheric presssure at outlet
p_out_tilde = 0;                                        % Adjusted pressure                                              
p_out = ones(1,M_total)*p_out_tilde;              % Outlet pressure profile

alpha_u = 0.005;                            % Under-relaxation factor for u  
alpha_v = 0.005;                            % Under-relaxation factor for v
alpha_p = 0.01;                             % Under-relaxation factor for p

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dimensionless parameters
mu = 1;                                           % Dimensionless viscosity
rho = 1;                                            % Dimensionless density
del_x = del_x_true/D_hyd;              % Dimensionless control volume width
del_y = del_y_true/D_hyd;             % Dimensionless control volume height
A_x = A_x_true/D_hyd^2; % Dimensionless cross-sectional area in x-direction
A_y = A_y_true/D_hyd^2; % Dimensionless cross-sectional area in y-direction
D_x = 1/Re*mu/del_x;   % Dimensionless diffusion conductance in x-direction
D_y = 1/Re*mu/del_y;   % Dimensionless diffusion conductance in y-direction
u_in = 1;                                                % Inlet u-velocity
v_in = 0;                                                % Inlet u-velocity
u_guess = 1.0;                               % Initial guess for u-velocity
v_guess = 0.0;                               % Initial guess for v-velocity
p_guess = 0/(rho_true*u_in_true^2);            % Initial guess for pressure 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation of p
% Filling in initial pressure vector with the linear profile.
% This section is set up for if gravity is added, but could be more compact
% if the option to add gravity was not there. 

p_circ_y_wide = linspace(p_guess, p_guess+rho*g_y*H_total,M_total);
p_circ_carthesian_wide = zeros(M_total,N_wide);
for j = 1:M_total
    for i = 1:N_wide
        p_circ_carthesian_wide(j,i) = p_circ_y_wide(j); 
    end %for
end %for

p_circ_y_narrow = p_circ_y_wide(M_wide+1:end);
p_circ_carthesian_narrow = zeros(M_narrow,N_narrow);
for j = 1:M_narrow
    for i = 1:N_narrow
        p_circ_carthesian_narrow(j,i) = p_circ_y_narrow(j); 
    end %for
end %for

filler = zeros(M_wide, N_narrow);
p_circ_carthesian = [[filler; p_circ_carthesian_narrow] ...
    p_circ_carthesian_wide ];
p_circ_carthesian = flip(p_circ_carthesian,1);

p_circ = p_circ_carthesian(1,:);                    % Take the first vector

for i = 2:M_total
    row = p_circ_carthesian(i);
    if i <= M_narrow                                       % Take whole row
        p_circ = [p_circ, p_circ_carthesian(i,:)];
    else                                             % Take part of the row
        p_circ = [p_circ, p_circ_carthesian(i,N_narrow+1:N_total)];
    end %if
end %for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation of u and v

u_circ = ones(totalpoints,1)*u_guess; % Fill in guess in the initial vector
if ~onlyChannel             % Only for the normal mode with the BFS enabled
    for i = 1:totalpoints
        if isWide(i, N_narrow, N_wide, M_wide)% Lower guess after expansion
            u_circ(i) = u_guess*(M_narrow/M_total); 
        end %if
    end %for
end %if


v_circ = ones(totalpoints_v,1)*v_guess; % Fill in guess in the initial vec.
if ~onlyChannel             % Only for the normal mode with the BFS enabled
    for i = 1:totalpoints_v
        if isWide(i, N_narrow, N_wide, M_wide)% Lower guess after expansion
            v_circ(i) = v_guess*(m_narrow/m_total);
        end %if 
    end %for
end %if

if plotInitialProfiles == true       % Plot the initial profiles if desired
    it = 0;
    u_new = u_circ;
    v_new = v_circ;
    p_new = p_circ;
    if printSetPlotIt == true
         plotProfilesITSAVE_subplots;
    else
        plotProfiles_dimensionless
        pause
        close all
    end % if
end %if

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation of solution vectors
p_new = zeros(1, totalpoints);                               % New pressure

u_corr = zeros(1, totalpoints);                     % u-velocity correction
u_new = zeros(1, totalpoints);                             % New u-velocity

v_corr = zeros(1, totalpoints_v);                   % v-velocity correction
v_new = zeros(1, totalpoints_v);                           % New v-velocity

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% While loop
conv = 0;                            % 0 is not converged, 1 when converged
it = 1;                                             % The current iteration

while  conv == 0 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  Calculate velocities and pressure correction
    % Run the scripts:
    % Velocities
    BFS_u_velocity
    BFS_v_velocity    
    if solvvel == false
        v_star = zeros(totalpoints_v,1);        
    end %if
   
    % Pressure correction
    BFS_pressurecorrection
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Velocity correction

    startCorr = 1;
    for j = startCorr:totalpoints
        if ( i <= N_wide*M_wide && mod(i, N_wide) == 0 ) ...   % Below step
             || ( i > N_wide*M_wide && mod(i-N_wide*M_wide, N_total) == 0) 
            % Eastern boundary : eastern pressure is known, no press. corr.
            u_corr(j) =  - A_x/au(j)*(-p_corr(j));  
        else
            u_corr(j) =  - A_x/au(j)*(p_corr(j+1)-p_corr(j));
        end % if
    end %for
            
    for k = startCorr:totalpoints_v
            v_corr(k) = - A_y/av(k)*...
                (p_corr(getRowOver(k, N_wide, M_wide, N_total))-p_corr(k));
    end %for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Under-relaxation
    
    u_new = alpha_u*(u_star + u_corr') + (1-alpha_u)*u_circ;
 
    if solvvel == false
        v_new = zeros(totalpoints_v,1);
    else
        v_new = alpha_v*(v_star + v_corr') + (1-alpha_v)*v_circ;
    end %if
    
    p_new = p_circ + alpha_p* p_corr';
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Check convergence 
    % Make sure there are no mistakes in the matrix operations above
    if ~isvector(u_new) || ~isvector(p_new) || ~isvector(p_new) 
        fprintf('u_new - %dx%d\n',size(u_new,1),size(u_new,2))
        fprintf('v_new - %dx%d\n',size(v_new,1),size(v_new,2))
        fprintf('p_new - %dx%d\n',size(p_new,1),size(p_new,2))
        error('Matrix addition gone wrong')        
    end
    
    if isnan(rcond(U)) || isnan(rcond(V)) || isnan(rcond(T))
%         clc                              % Remove if warnings are desired
        fprintf('Stopped due to singularity in matrix\n')
        fprintf('RCOND u-velocity: %e \nRCOND v-velocity: %e \n',...
            rcond(U), rcond(V))
        fprintf('RCOND pressure correction: %e\n',rcond(T))
        fprintf('Problem occured after %d iterations\n', it)
        toc
        return
    end %if
        
    c1 = 1/u_in*sqrt((U*u_star-bu')'*(U*u_star-bu'));           % residuals 
    c2 = 1/u_in*sqrt((V*v_star-bv')'*(V*v_star-bv'));           % residuals
    c3 = abs(sum(beta));                             % continuity fulfulled
    c4 = 1/u_in*max(abs(u_circ - u_star)) ;    % change from last iteration
    c5 = 1/u_in*max(abs(v_circ - v_star))  ;   % change from last iteration

    c1_lim = 10^-8;                                                % Limits
    c2_lim = 10^-8;
    c3_lim = 10^-10;
    c4_lim = 10^-8;
    c5_lim = 10^-8;
    
    if solvvel == false         % Overwrite if v-velocity is not solved for
        c2 = 0;
        c5 = 0;
    end %if
    
    c1_diff = c1-c1_lim;                    % How far away from convergence
    c2_diff = c2-c2_lim;
    c3_diff = c3-c3_lim;
    c4_diff = c4-c4_lim;
    c5_diff = c5-c5_lim;
    
    
   if  (c1 < c1_lim) && (c2 < c2_lim) && (c3 < c3_lim) && (c4 < c4_lim) ...
            && (c5 < c5_lim) || (it == maxits) 
        conv = 1;                                               % Converged
        if (it == maxits)
            fprintf('Stopped at max iterations (%d)\n',it);
        else
            fprintf('Solution converged after %d iterations\n',it);
        end %if
    
        fprintf('c1\tMomentum residual u\t\t%.2e\tLimit: %.2e\n',...
            c1,c1_lim);
        fprintf('c2\tMomentum residual v\t\t%.2e\tLimit: %.2e\n',...
            c2,c2_lim);
        fprintf('c3\tPressure correction\t\t%.2e\tLimit: %.2e\n',...
            c3,c3_lim);
        fprintf('c4\tDiff. last iteration u\t%.2e\tLimit: %.2e\n',...
            c4,c4_lim); 
        fprintf('c5\tDiff. last iteration v\t%.2e\tLimit: %.2e\n',...
            c5,c5_lim);              

        if max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c1_diff
            fprintf('Limiting criteria is c1\tMomentum residual u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c2_diff
            fprintf('Limiting criteria is c2\tMomentum residual v\n')            
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c3_diff
            fprintf('Limiting criteria is c3\tPressure correction\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c4_diff
            fprintf('Limiting criteria is c4\tDiff. last iteration u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c5_diff
            fprintf('Limiting criteria is c5\tDiff. last iteration u\n')            
        end %if
        
        showStep = false;    
        if plotCircVels == true
            plotIntermediates
        end
        if plotCorrVels == true
            plotVelocityCorrection
        end
        plot_BFS
        if showVelociyQuiver == true
            plotVelocityQuiver
            plotColoredQuiver
        end %if        
        
    else
        if plotiterationwise == true
            showStep = false;            
            if plotCircVels == true
                plotIntermediates
            end
            if plotCorrVels == true
                plotVelocityCorrection
            end
            plot_BFS
            if showVelociyQuiver == true
                plotVelocityQuiver
                plotColoredQuiver
            end %if
            pause
            close all 
        end %if
        if printSetPlotIt && ismember(it,itSaves) 
            plot_BFS_iterations
            if gifIntermediates == true
                plotVelInts_BFS_iterations;
            end %if
        end
       
        u_circ = u_new;                  % Not converged, updated variables
        v_circ = v_new;                  % Not converged, updated variables
        p_circ = p_new;                  % Not converged, updated variables

        it = it + 1;  % Update number of iterations
    end % if
end %while
toc