%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Two dimensional fluid flow over a backwards facing step, dimensionless  %
%             Model adjusted to Reynolds number comparison                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear
clc
tic
warning on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Solver specs
maxits = 50000;   % Maximum number of iterations, stop if iterations exceed
% Choose which inlet profile to use
run('inletprofileRe400.m')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% System specifications
% Specify number of narrow points, leave the rest

N_narrow = 10;  % Number of scalar nodal points in narrow section in x-dir. 
M_narrow = 10;  % Number of scalar nodal points in narrow section in x-dir. 

l = 5;                                              % Narrow channel length 
h = 1;                                              % Narrow channel height
L = 30;                                               % Wide channel length
H = 1;                                                % Wide channel height

L_total = l + L;                                     % Total channel length
H_total = h + H;                                     % Total channel height

x_0 = 0;                                % Defining the domain using x and y
x_N = L_total;
y_0 = 0;
y_M = H_total;

N_wide = N_narrow*L/l;    % # scalar nodal points in wide section in x-dir.
M_wide = M_narrow*H/h;    % # scalar nodal points in wide section in y-dir.

% For extension to the wide channel the number of nodes in the narrow
% section needs to meet these criteria:
if floor(N_narrow)~= N_narrow || floor(N_wide)~= N_wide|| ...
        floor(M_narrow)~= M_narrow || floor(M_wide)~= M_wide
    msg = 'Points don''t match dimensions';
    error(msg)
end %if 
N_total = N_narrow + N_wide;% Total # of scalar nodal points in x-direction 
M_total = M_narrow + M_wide;% Total # of scalar nodal points in y-direction 

m_total = M_total - 1;    % Total number of y-velocity nodes in y-direction
m_wide = M_wide;% Number of y-velocity nodes in y-direction in wide section
m_narrow = M_narrow - 1;% # of y-velocity nodes in y-dir. in narrow section

% Total number of computational points in the domain ...
totalpoints = N_narrow*M_narrow + N_wide*M_total;        %  ... for u and P
totalpoints_v = N_narrow*m_narrow + N_wide*m_total;             % ... for v

D_hyd = 2*h;                                           % Hydraulic diameter
mu_true = 8.90 * 10^-4;                                % Viscosity of water

del_z_true = 1;                                              % System depth
del_x_true = L_total/N_total;                        % Control volume width
del_y_true = H_total/M_total;                       % Control volume height
A_x_true = del_y_true*del_z_true;     % Cross-sectional area in x-direction
A_y_true = del_x_true*del_z_true;     % Cross-sectional area in y-direction

rho_true = 997;                                          % Density of water    
u_in_true = u_bulk;                                      % Inlet u-velocity

g_x = 0;                                                   % No gravitation
g_y = 0;                                                   % No gravitation

Re = rho_true*D_hyd*u_in_true/mu_true;                    % Reynolds number

p_atm = 101325;                           % Atmospheric presssure at outlet
p_out_tilde = 0;                                        % Adjusted pressure                                              
p_out = ones(1,M_total)*p_out_tilde;              % Outlet pressure profile

alpha_u = 0.01;                             % Under-relaxation factor for u 
alpha_v = 0.01;                             % Under-relaxation factor for v
alpha_p = 0.02;                             % Under-relaxation factor for p
alpha_u = 0.005;                             % Under-relaxation factor for u 
alpha_v = 0.005;                             % Under-relaxation factor for v
alpha_p = 0.01;                             % Under-relaxation factor for p

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dimensionless parameters
mu = 1;                                           % Dimensionless viscosity
rho = 1;                                            % Dimensionless density
del_x = del_x_true/h;              % Dimensionless control volume width
del_y = del_y_true/h;             % Dimensionless control volume height
A_x = A_x_true/h^2; % Dimensionless cross-sectional area in x-direction
A_y = A_y_true/h^2; % Dimensionless cross-sectional area in y-direction
D_x = 2/Re*mu/del_x;   % Dimensionless diffusion conductance in x-direction
D_y = 2/Re*mu/del_y;   % Dimensionless diffusion conductance in y-direction
u_in = u_in/u_in_true;                                   % Inlet u-velocity
u_bulk_dimless = u_bulk/u_in_true;       % Bulk inlet velocity (which is 1)
v_in = 0;                                                % Inlet u-velocity
u_guess = u_max;                             % Initial guess for u-velocity
v_guess = 0.0;                               % Initial guess for v-velocity
p_guess = 0/(rho_true*u_in_true^2);            % Initial guess for pressure 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation of p
% Filling in initial pressure vector with the linear profile.
% This section is set up for if gravity is added, but could be more compact
% if the option to add gravity was not there. 

p_circ_y_wide = linspace(p_guess, p_guess+rho*g_y*H_total,M_total);
p_circ_carthesian_wide = zeros(M_total,N_wide);
for j = 1:M_total
    for i = 1:N_wide
        p_circ_carthesian_wide(j,i) = p_circ_y_wide(j); 
    end %for
end %for

p_circ_y_narrow = p_circ_y_wide(M_wide+1:end);
p_circ_carthesian_narrow = zeros(M_narrow,N_narrow);
for j = 1:M_narrow
    for i = 1:N_narrow
        p_circ_carthesian_narrow(j,i) = p_circ_y_narrow(j); 
    end %for
end %for

filler = zeros(M_wide, N_narrow);
p_circ_carthesian = [[filler; p_circ_carthesian_narrow] ...
    p_circ_carthesian_wide ];
p_circ_carthesian = flip(p_circ_carthesian,1);

p_circ = p_circ_carthesian(1,:);                    % Take the first vector

for i = 2:M_total
    row = p_circ_carthesian(i);
    if i <= M_narrow                                       % Take whole row
        p_circ = [p_circ, p_circ_carthesian(i,:)];
    else                                             % Take part of the row
        p_circ = [p_circ, p_circ_carthesian(i,N_narrow+1:N_total)];
    end %if
end %for


%% Initialisation of u and v

u_circ = ones(totalpoints,1)*u_guess; % Fill in guess in the initial vector
for i = 1:totalpoints
    if isWide(i, N_narrow, N_wide, M_wide)% Lower guess after expansion
        u_circ(i) = u_guess*(M_narrow/M_total); 
    end %if
end %for


v_circ = ones(totalpoints_v,1)*v_guess; % Fill in guess in the initial vec.
for i = 1:totalpoints_v
    if isWide(i, N_narrow, N_wide, M_wide)% Lower guess after expansion
        v_circ(i) = v_guess*(m_narrow/m_total);
    end %if 
end %for


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialisation of solution vectors
p_new = zeros(1, totalpoints);                               % New pressure

u_corr = zeros(1, totalpoints);                     % u-velocity correction
u_new = zeros(1, totalpoints);                             % New u-velocity

v_corr = zeros(1, totalpoints_v);                   % v-velocity correction
v_new = zeros(1, totalpoints_v);                           % New v-velocity

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% While loop
conv = 0;                            % 0 is not converged, 1 when converged
it = 1;                                             % The current iteration

while  conv == 0 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  Calculate velocities and pressure correction
    % Run the scripts:
    % Velocities
    BFS_u_velocity_parabolic
    BFS_v_velocity_parabolic  
   
    % Pressure correction
    BFS_pressurecorrection_parabolic
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Velocity correction

    startCorr = 1;
    for j = startCorr:totalpoints
        if ( i <= N_wide*M_wide && mod(i, N_wide) == 0 ) ...   % Below step
             || ( i > N_wide*M_wide && mod(i-N_wide*M_wide, N_total) == 0) 
            % Eastern boundary : eastern pressure is known, no press. corr.
            u_corr(j) =  - A_x/au(j)*(-p_corr(j));  
        else
            u_corr(j) =  - A_x/au(j)*(p_corr(j+1)-p_corr(j));
        end % if
    end %for
            
    for k = startCorr:totalpoints_v
            v_corr(k) = - A_y/av(k)*...
                (p_corr(getRowOver(k, N_wide, M_wide, N_total))-p_corr(k));
    end %for
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Under-relaxation
    
    u_new = alpha_u*(u_star + u_corr') + (1-alpha_u)*u_circ;
 
    v_new = alpha_v*(v_star + v_corr') + (1-alpha_v)*v_circ;
    
    p_new = p_circ + alpha_p* p_corr';
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Check convergence 
    % Make sure there are no mistakes in the matrix operations above
    if ~isvector(u_new) || ~isvector(p_new) || ~isvector(p_new) 
        fprintf('u_new - %dx%d\n',size(u_new,1),size(u_new,2))
        fprintf('v_new - %dx%d\n',size(v_new,1),size(v_new,2))
        fprintf('p_new - %dx%d\n',size(p_new,1),size(p_new,2))
        error('Matrix addition gone wrong')        
    end
    
    if isnan(rcond(U)) || isnan(rcond(V)) || isnan(rcond(T))
%         clc                              % Remove if warnings are desired
        fprintf('Stopped due to singularity in matrix\n')
        fprintf('RCOND u-velocity: %e \nRCOND v-velocity: %e \n',...
            rcond(U), rcond(V))
        fprintf('RCOND pressure correction: %e\n',rcond(T))
        fprintf('Problem occured after %d iterations\n', it)
        toc
        return
    end %if
        
    c1 = 1/u_bulk_dimless*sqrt((U*u_star-bu')'*(U*u_star-bu'));           % residuals 
    c2 = 1/u_bulk_dimless*sqrt((V*v_star-bv')'*(V*v_star-bv'));           % residuals
    c3 = abs(sum(beta));                             % continuity fulfulled
    c4 = 1/u_bulk_dimless*max(abs(u_circ - u_star)) ;    % change from last iteration
    c5 = 1/u_bulk_dimless*max(abs(v_circ - v_star))  ;   % change from last iteration

    c1_lim = 10^-8;                                                % Limits
    c2_lim = 10^-8;
    c3_lim = 10^-10;
    c4_lim = 10^-8;
    c5_lim = 10^-8;
    
    c1_diff = c1-c1_lim;                    % How far away from convergence
    c2_diff = c2-c2_lim;
    c3_diff = c3-c3_lim;
    c4_diff = c4-c4_lim;
    c5_diff = c5-c5_lim;
    
    
   if  (c1 < c1_lim) && (c2 < c2_lim) && (c3 < c3_lim) && (c4 < c4_lim) ...
            && (c5 < c5_lim) || (it == maxits) 
        conv = 1;                                               % Converged
        if (it == maxits)
            fprintf('Stopped at max iterations (%d)\n',it);
        else
            fprintf('Solution converged after %d iterations\n',it);
        end %if
    
        fprintf('c1\tMomentum residual u\t\t%.2e\tLimit: %.2e\n',...
            c1,c1_lim);
        fprintf('c2\tMomentum residual v\t\t%.2e\tLimit: %.2e\n',...
            c2,c2_lim);
        fprintf('c3\tPressure correction\t\t%.2e\tLimit: %.2e\n',...
            c3,c3_lim);
        fprintf('c4\tDiff. last iteration u\t%.2e\tLimit: %.2e\n',...
            c4,c4_lim); 
        fprintf('c5\tDiff. last iteration v\t%.2e\tLimit: %.2e\n',...
            c5,c5_lim);              

        if max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c1_diff
            fprintf('Limiting criteria is c1\tMomentum residual u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c2_diff
            fprintf('Limiting criteria is c2\tMomentum residual v\n')            
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c3_diff
            fprintf('Limiting criteria is c3\tPressure correction\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c4_diff
            fprintf('Limiting criteria is c4\tDiff. last iteration u\n')
        elseif max([c1_diff c2_diff c3_diff c4_diff c5_diff])== c5_diff
            fprintf('Limiting criteria is c5\tDiff. last iteration u\n')            
        end %if
     
        plotColoredQuiver_parabolic
                  
   else
       
        u_circ = u_new;                  % Not converged, updated variables
        v_circ = v_new;                  % Not converged, updated variables
        p_circ = p_new;                  % Not converged, updated variables

        it = it + 1;  % Update number of iterations
    end % if
end %while
toc