%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Transfinite Interpolation                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clc
%% Settings 
N = 71; % Number of points in q1/x-direction
M = 21; % Number of points in q2/y-direction

x_max = 35; % Total length of physical domain (including step)
y_max = 2; % Total height of physical domain (including step)

h = 1; % Height of the step
l = 5; % Length / width of the step

% Placement of points E and F splits the line segment AD in s equal pieces.
s = 3;


%% Boundary points
q1 = 0:N; % Specifying the q1-points with spacing of delta q1 = 1
q2 = 0:M; % Specifying the q1-points with spacing of delta q1 = 1

% Specifying the locations of points A-F in the physical domain
xA = 0;
xB = 0;
xC = x_max;
xD = x_max;
xE = l;
xF = l;

yA = h;
yB = y_max;
yC = y_max;
yD = 0;
yE = 0;
yF = h;

% Place points E and F to split the line segment AD in s equal pieces.
AFfrac = 1/s; % Fraction of total width of q1
AEfrac = 1/s; % Fraction of total width of q1
AFpoints = ceil(AFfrac * N); % Number of q1-points in line segment AF
FEpoints = floor(AEfrac * N); % Number of q1-points in line segment FE

q1AF = 0:AFpoints; % Vector of coordinates q1 for the line segment AF
q1FE = 0:FEpoints; % Vector of coordinates q1 for the line segment FE
q1ED = 0:(N-AFpoints-FEpoints); % Vector of coordinates q1 for ...
                                % the line segment ED

% Calculation of the boundary points:                                                               
xAB = (1-q2/q2(end))* xA + q2/q2(end)*xB;
xBC = (1-q1/q1(end))* xB + q1/q1(end)*xC;
xDC = (1-q2/q2(end))* xD + q2/q2(end)*xC;
xED = (1-q1ED/q1ED(end))* xE + q1ED/q1ED(end)*xD;
xFE = (1-q1FE/q1FE(end))* xF + q1FE/q1FE(end)*xE;
xAF = (1-q1AF/q1AF(end))* xA + q1AF/q1AF(end)*xF;

yAB = (1-q2/q2(end))* yA + q2/q2(end)*yB;
yBC = (1-q1/q1(end))* yB + q1/q1(end)*yC;
yDC = (1-q2/q2(end))* yD + q2/q2(end)*yC;
yED = (1-q1ED/q1ED(end))* yE + q1ED/q1ED(end)*yD;
yFE = (1-q1FE/q1FE(end))* yF + q1FE/q1FE(end)*yE;
yAF = (1-q1AF/q1AF(end))* yA + q1AF/q1AF(end)*yF;

% Plot with the boundary points
% figure
% plot(xAB,yAB,'x',xBC,yBC,'x',xDC,yDC,'x',...
%     xED,yED,'x',xFE,yFE,'x',xAF,yAF,'x')
% % xlim([-0.1,1.1])
% % ylim([-0.1,1.1])
% legend({'$AB$','$BC$','$CD$','$DE$','$EF$','$FA$'},...
%     'Interpreter','latex','Location','best')

%% Center domain points
% Combining the x- and y-points for the line segments AF, FE and ED to ...
% one vector for AD. The points located exactly at F and E are ...
% overlapping and removed from xFE by taking xFE(2:end-1). Likewise for y.

xAD = [xAF xFE(2:end-1) xED];% Combining the x-points for the line segment
yAD = [yAF yFE(2:end-1) yED];% Combining the y-points for the line segment

% Initialising the matrix x of points in the physical domain
x = zeros(length(q2),length(q1)); 
% Initialising the matrix y of points in the physical domain
y = zeros(length(q2),length(q1));

% Calculating the center points
for j =1:length(q2)
    for i = 1:length(q1)
        x(j,i) = (1-q1(i)/q1(end))* xAB(j) +(q1(i)/q1(end)) *xDC(j)...
            +(1-q2(j)/q2(end))*xAD(i) +(q2(j)/q2(end))* xBC(i)...
            -(1-q1(i)/q1(end))* (1-q2(j)/q2(end))* xA...
            -(1-q1(i)/q1(end))* (q2(j)/q2(end))* xB...
            -(q1(i)/q1(end))*(1-q2(j)/q2(end))* xD...
            -(q1(i)/q1(end))*(q2(j)/q2(end))* xC;
        y(j,i) = (1-q1(i)/q1(end))* yAB(j) +(q1(i)/q1(end)) *yDC(j)...
            +(1-q2(j)/q2(end))*yAD(i) +(q2(j)/q2(end))* yBC(i)...
            -(1-q1(i)/q1(end))* (1-q2(j)/q2(end))* yA...
            -(1-q1(i)/q1(end))* (q2(j)/q2(end))* yB...
            -(q1(i)/q1(end))*(1-q2(j)/q2(end))* yD...
            -(q1(i)/q1(end))*(q2(j)/q2(end))* yC;
    end %for
end %for

% Plotting the resulting grid
figure 
plot(x,y,'k',x',y','k')
xlim([xA,xD])
ylim([yD,yC])
set(gca,'TickLabelInterpreter','latex')
xlabel('$x$-direction [m]', 'interpreter', 'latex')
ylabel('$y$-direction [m]', 'interpreter', 'latex')
saveas(gcf,'transfinite.png')




